#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "MenuV3.h"

#define LINEFEED 10

MenuItem *newMenuItem(MenuText text)
{
    MenuItem *item;
    assert(text != NULL);   //Precondizione
    item = (MenuItem*)malloc(sizeof(MenuItem));
    strcpy(item->text, text);
    item->next = NULL;
    item->subItem = NULL;
    return item;
}

void destroyMenuItem(MenuItem *menu)
{
    assert(menu != NULL);   //Precondizione
    if (menu->subItem != NULL)
        destroyMenuItem(menu->subItem);
    if (menu->next != NULL)
        destroyMenuItem(menu->next);
    free(menu);
}

void appendItem(MenuItem *menu, MenuItem *toAppend)
{
    assert(menu != NULL && toAppend != NULL);   //Precondizione
    if (menu->next != NULL)
    {
        appendItem(menu->next, toAppend);
    }
    else
    {
        menu->next = toAppend;
    }
}

void insertBeforeItem(MenuItem *menu, MenuItem *reference, MenuItem *toInsert)
{
    assert(menu != NULL && menu->next != NULL && reference != NULL && toInsert != NULL);    //Precondizione
    if (menu->next != reference)
    {
        insertBeforeItem(menu->next, reference, toInsert);
    }
    else
    {
        toInsert->next = reference;
        menu->next = toInsert;
    }
}

MenuItem* getNextItem(MenuItem *item)
{
    assert(item != NULL);   //Precondizione
    return item->next;
}

MenuItem* getSubMenu(MenuItem *item)
{
    assert(item != NULL);   //Precondizione
    return item->subItem;
}

void appendSubItem(MenuItem *menu, MenuItem *toAppend)
{
    assert(menu != NULL && toAppend != NULL);   //Precondizione
    if (menu->subItem != NULL)
        appendItem(menu->subItem, toAppend);
    else
        menu->subItem = toAppend;
}

void insertBeforeSubItem(MenuItem *menu, MenuItem *reference, MenuItem *toInsert)
{
    assert(menu != NULL && menu->subItem != NULL && toInsert != NULL);  //Precondizione

    if (menu->subItem != reference)
    {
        insertBeforeItem(menu->subItem, reference, toInsert);
    }
    else
    {
        toInsert->next = reference;
        menu->subItem = toInsert;
    }
}

int itemCount(MenuItem *item)
{
    if (item != NULL)
        return itemCount(item->next) + 1;
    return 0;
}

int subItemCount(MenuItem *item)
{
    assert(item != NULL);   //Precondizione
    if (item->subItem != NULL)
        return itemCount(item->subItem);
    return 0;
}

int render(MenuItem *menu, char title[])
{
    int choiceId;
    assert(menu != NULL && title != NULL);  //Precondizione
    system("cls");
    printf("********************************************************************************");
    printf("     %s\n", title);
    printf("********************************************************************************\n\n");
    for (choiceId = 1; menu != NULL; choiceId++, menu = menu->next)
        printf("    %2d - %s\n", choiceId, menu->text);
    printf("    %2d - %s\n\n", 0, "Esci");
    printf("    Opzione scelta: ");
    return choiceId - 1;
}

MenuItem* getItemFromChoiceId(MenuItem *menu, int choiceId)
{
    int currentId;
    assert(menu != NULL);   //Precondizione
    for (currentId = 1; menu != NULL && choiceId != currentId; menu = menu->next, currentId++);
    return menu;
}

int readUserChoiceId(MenuItem *menu)
{
    int paramsRead, choiceId, count;
    assert(menu != NULL);   //Precondizione
    count = itemCount(menu);
    do
    {
        choiceId = 0;
        paramsRead = scanf("%d", &choiceId);
        while (getchar() != LINEFEED);  //Mangia fino all'LF
        if (paramsRead == 0 || choiceId < 0 || choiceId > count)
            printf("Digitare un valore fra 0 e %d\n", count);
    }
    while (paramsRead == 0 || choiceId < 0 || choiceId > count);
    assert(paramsRead == 1);
    return choiceId;
}

UserChoice* showMenu(MenuItem *menu, char title[])
{
    int choiceId;
    MenuItem *chosenItem;
    UserChoice *choice = NULL, *subChoice;
    assert(menu != NULL && title != NULL);  //Precondizione
    do
    {
        choice = NULL;
        chosenItem = NULL;
        subChoice = NULL;
        render(menu, title);
        choiceId = readUserChoiceId(menu);
        chosenItem = getItemFromChoiceId(menu, choiceId);
        if (chosenItem != NULL)
        {
            if (chosenItem->subItem != NULL)
            {
                subChoice = showMenu(chosenItem->subItem, chosenItem->text);
                if (subChoice != NULL)
                {   
                    choice = newUserChoice(choiceId, chosenItem);
                    appendChoice(choice, subChoice);
                }
            }
            else
            {
                choice = newUserChoice(choiceId, chosenItem);
            }
        }
    } while (chosenItem != NULL && chosenItem->subItem != NULL && subChoice == NULL);
    return choice;
}

UserChoice* newUserChoice(int choiceId, MenuItem *item)
{
    UserChoice *choice;
    assert (item != NULL);  //Precondizione
    choice = (UserChoice*)malloc(sizeof(UserChoice));
    choice->choiceId = choiceId;
    choice->item = item;
    choice->next = NULL;
    return choice;
}

void destroyUserChoice(UserChoice *choice)
{
    assert(choice != NULL); //Precondizione
    if (choice->next != NULL)
        destroyUserChoice(choice->next);
    free(choice);
}

void appendChoice(UserChoice *choice, UserChoice *toAppend)
{
    assert(choice != NULL && toAppend != NULL); //Precondizione
    choice->next = toAppend;
}

void testMenuV3()
{
    UserChoice *choice, *choiceNav;
    MenuItem *m1, *m2, *m3, *m4;
    MenuItem *m11, *m12, *m13;
    MenuItem *m121, *m122;
    MenuItem *m2Ins, *m12Ins;

    m1 = newMenuItem("Uno");
    m2 = newMenuItem("Due");
    m3 = newMenuItem("Tre");
    m4 = newMenuItem("Quattro");
    m2Ins = newMenuItem("Due.Ins");
    appendItem(m1, m2);
    appendItem(m1, m3);
    appendItem(m1, m4);
    insertBeforeItem(m1, m3, m2Ins);

    m11 = newMenuItem("Uno - Uno");
    m12 = newMenuItem("Uno - Due");
    m13 = newMenuItem("Uno - Tre");
    m12Ins = newMenuItem("Uno - Due.Ins");
    appendSubItem(m1, m11);
    appendSubItem(m1, m12);
    appendSubItem(m1, m13);
    insertBeforeSubItem(m1, m13, m12Ins);

    m121 = newMenuItem("Uno - Due - Uno");
    m122 = newMenuItem("Uno - Due - Due");
    appendSubItem(m12, m121);
    appendSubItem(m12, m122);

    choice = showMenu(m1, "Principale");
    printf("\n\n\n\n");
    choiceNav = choice;
    while(choiceNav != NULL)
    {
        printf("-> %d %s", choiceNav->choiceId, choiceNav->item->text);
        choiceNav = choiceNav->next;
    }

    if (choice != NULL)
        destroyUserChoice(choice);
    destroyMenuItem(m1);
}
